@@ -39,6 +39,7 @@ pub(crate) enum Menu {
 
 pub(crate) struct PendingMenu {
     pub menu: Menu,
+    pub is_hidden: bool,
     pub(crate) args: BTreeMap<Cow<'static, str>, arg::Arg>,
 }
 
@@ -46,22 +47,23 @@ impl PendingMenu {
     pub fn init(menu: Menu) -> Self {
         Self {
             menu,
+            is_hidden: false,
             args: match menu {
-                Menu::Root => &[],
-                Menu::Branch => ops::checkout::ARGS,
-                Menu::Commit => ops::commit::ARGS,
-                Menu::Fetch => ops::fetch::ARGS,
-                Menu::Help => &[],
-                Menu::Log => ops::log::ARGS,
-                Menu::Pull => ops::pull::ARGS,
-                Menu::Push => ops::push::ARGS,
-                Menu::Rebase => ops::rebase::ARGS,
-                Menu::Reset => ops::reset::ARGS,
-                Menu::Revert => ops::revert::ARGS,
-                Menu::Stash => ops::stash::ARGS,
+                Menu::Root => vec![],
+                Menu::Branch => ops::checkout::init_args(),
+                Menu::Commit => ops::commit::init_args(),
+                Menu::Fetch => ops::fetch::init_args(),
+                Menu::Help => vec![],
+                Menu::Log => ops::log::init_args(),
+                Menu::Pull => ops::pull::init_args(),
+                Menu::Push => ops::push::init_args(),
+                Menu::Rebase => ops::rebase::init_args(),
+                Menu::Reset => ops::reset::init_args(),
+                Menu::Revert => ops::revert::init_args(),
+                Menu::Stash => ops::stash::init_args(),
             }
-            .iter()
-            .map(|arg| (Cow::from(arg.arg), arg.clone()))
+            .into_iter()
+            .map(|arg| (Cow::from(arg.arg), arg))
             .collect(),
         }
     }
@@ -69,8 +71,8 @@ impl PendingMenu {
     pub fn args(&self) -> Vec<OsString> {
         self.args
             .iter()
-            .filter(|&(_k, arg)| arg.state)
-            .map(|(k, _v)| k.to_string().into())
+            .filter(|&(_k, arg)| arg.is_active())
+            .map(|(_, v)| v.get_cli_token().into())
             .collect()
     }
 }
