@@ -1,6 +1,5 @@
 use super::{create_prompt_with_default, selected_rev, Action, OpTrait};
 use crate::{items::TargetData, menu::arg::Arg, prompt::PromptData, state::State, term::Term, Res};
-use derive_more::Display;
 use std::{process::Command, rc::Rc};
 use tui_prompts::State as _;
 
@@ -8,8 +7,6 @@ pub(crate) fn init_args() -> Vec<Arg> {
     vec![]
 }
 
-#[derive(Display)]
-#[display(fmt = "Checkout branch/revision")]
 pub(crate) struct Checkout;
 impl OpTrait for Checkout {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
@@ -20,6 +17,10 @@ impl OpTrait for Checkout {
             true,
         ))
     }
+
+    fn display(&self, _state: &State) -> String {
+        "Checkout branch/revision".into()
+    }
 }
 
 fn checkout(state: &mut State, term: &mut Term, rev: &str) -> Res<()> {
@@ -33,8 +34,6 @@ fn checkout(state: &mut State, term: &mut Term, rev: &str) -> Res<()> {
     Ok(())
 }
 
-#[derive(Display)]
-#[display(fmt = "Checkout new branch")]
 pub(crate) struct CheckoutNewBranch;
 impl OpTrait for CheckoutNewBranch {
     fn get_action(&self, _target: Option<&TargetData>) -> Option<Action> {
@@ -47,6 +46,10 @@ impl OpTrait for CheckoutNewBranch {
             Ok(())
         }))
     }
+
+    fn display(&self, _state: &State) -> String {
+        "Checkout new branch".into()
+    }
 }
 
 fn checkout_new_branch_prompt_update(state: &mut State, term: &mut Term) -> Res<()> {
