@@ -27,7 +27,7 @@ pub(crate) mod unstage;
 
 pub(crate) type Action = Rc<dyn FnMut(&mut State, &mut Term) -> Res<()>>;
 
-pub(crate) trait OpTrait: Display {
+pub(crate) trait OpTrait {
     /// Get the implementation (which may or may not exist) of the Op given some TargetData.
     /// This indirection allows Gitu to show a contextual menu of applicable actions.
     fn get_action(&self, target: Option<&TargetData>) -> Option<Action>;
@@ -37,6 +37,8 @@ pub(crate) trait OpTrait: Display {
     fn is_target_op(&self) -> bool {
         false
     }
+
+    fn display(&self, state: &State) -> String;
 }
 
 #[derive(Clone, PartialOrd, Ord, PartialEq, Eq, Debug, Serialize, Deserialize)]
@@ -49,10 +51,12 @@ pub(crate) enum Op {
     FetchAll,
     FetchElsewhere,
     LogCurrent,
-    Pull,
-    PullElsewhere,
-    Push,
-    PushElsewhere,
+    PullFromPushRemote,
+    PullFromUpstream,
+    PullFromElsewhere,
+    PushToPushRemote,
+    PushToUpstream,
+    PushToElsewhere,
     RebaseAbort,
     RebaseContinue,
     RebaseElsewhere,
@@ -127,10 +131,12 @@ impl Op {
             Op::FetchAll => Box::new(fetch::FetchAll),
             Op::FetchElsewhere => Box::new(fetch::FetchElsewhere),
             Op::LogCurrent => Box::new(log::LogCurrent),
-            Op::Pull => Box::new(pull::Pull),
-            Op::PullElsewhere => Box::new(pull::PullElsewhere),
-            Op::Push => Box::new(push::Push),
-            Op::PushElsewhere => Box::new(push::PushElsewhere),
+            Op::PullFromPushRemote => Box::new(pull::PullFromPushRemote),
+            Op::PullFromUpstream => Box::new(pull::PullFromUpstream),
+            Op::PullFromElsewhere => Box::new(pull::PullFromElsewhere),
+            Op::PushToPushRemote => Box::new(push::PushToPushRemote),
+            Op::PushToUpstream => Box::new(push::PushToUpstream),
+            Op::PushToElsewhere => Box::new(push::PushToElsewhere),
             Op::RebaseAbort => Box::new(rebase::RebaseAbort),
             Op::RebaseContinue => Box::new(rebase::RebaseContinue),
             Op::RebaseElsewhere => Box::new(rebase::RebaseElsewhere),
@@ -214,48 +220,38 @@ pub(crate) fn create_y_n_prompt(mut action: Action, prompt: &'static str) -> Act
 
 pub(crate) fn create_prompt(
     prompt: &'static str,
-    callback: fn(&mut State, &mut Term, &str) -> Res<()>,
+    on_success: fn(&mut State, &mut Term, &str) -> Res<()>,
     hide_menu: bool,
 ) -> Action {
-    create_prompt_with_default(prompt, callback, |_| None, hide_menu)
+    create_prompt_with_default(prompt, on_success, |_| None, hide_menu)
 }
 
 pub(crate) fn create_prompt_with_default(
     prompt: &'static str,
-    callback: fn(&mut State, &mut Term, &str) -> Res<()>,
-    default_fn: fn(&State) -> Option<String>,
+    on_success: fn(&mut State, &mut Term, &str) -> Res<()>,
+    create_default_value: fn(&State) -> Option<String>,
     hide_menu: bool,
 ) -> Action {
     Rc::new(move |state: &mut State, _term: &mut Term| {
         set_prompt(
             state,
             prompt,
-            invoke_default,
-            Box::new(default_fn),
-            callback,
+            Box::new(on_success),
+            Box::new(create_default_value),
             hide_menu,
         );
         Ok(())
     })
 }
 
-fn invoke_default(
-    state: &mut State,
-    term: &mut Term,
-    value: &str,
-    context: &fn(&mut State, &mut Term, &str) -> Res<()>,
-) -> Res<()> {
-    context(state, term, value)
-}
-
 type DefaultFn = Box<dyn Fn(&State) -> Option<String>>;
+type PromptAction = Box<dyn Fn(&mut State, &mut Term, &str) -> Res<()>>;
 
-pub(crate) fn set_prompt<T: 'static>(
+pub(crate) fn set_prompt(
     state: &mut State,
     prompt: &'static str,
-    callback: fn(&mut State, &mut Term, &str, &T) -> Res<()>,
+    on_success: PromptAction,
     default_fn: DefaultFn,
-    context: T,
     hide_menu: bool,
 ) {
     let prompt_text = if let Some(default) = default_fn(state) {
@@ -282,7 +278,7 @@ pub(crate) fn set_prompt<T: 'static>(
                     (value, _) => value,
                 };
 
-                callback(state, term, value, &context)?;
+                on_success(state, term, value)?;
 
                 if hide_menu {
                     state.unhide_menu();
