@@ -1,6 +1,7 @@
 use super::Screen;
 use crate::{
     config::Config,
+    error::Error,
     git::{self, diff::Diff},
     git2_opts,
     items::{self, Item, TargetData},
@@ -19,7 +20,9 @@ pub(crate) fn create(config: Rc<Config>, repo: Rc<Repository>, size: Size) -> Re
         size,
         Box::new(move || {
             let style = &config.style;
-            let statuses = repo.statuses(Some(&mut git2_opts::status(&repo)?))?;
+            let statuses = repo
+                .statuses(Some(&mut git2_opts::status(&repo)?))
+                .map_err(Error::GitStatus)?;
 
             let untracked_files = statuses
                 .iter()
@@ -27,14 +30,7 @@ pub(crate) fn create(config: Rc<Config>, repo: Rc<Repository>, size: Size) -> Re
                 .map(|status| PathBuf::from(status.path().unwrap()))
                 .collect::<Vec<_>>();
 
-            let unmerged_files = statuses
-                .iter()
-                .filter(|status| status.status().is_conflicted())
-                .map(|status| PathBuf::from(status.path().unwrap()))
-                .collect::<Vec<_>>();
-
             let untracked = items_list(&config, untracked_files.clone());
-            let unmerged = items_list(&config, unmerged_files);
 
             let items = if let Some(rebase) = git::rebase_status(&repo)? {
                 vec![Item {
@@ -85,32 +81,17 @@ pub(crate) fn create(config: Rc<Config>, repo: Rc<Repository>, size: Size) -> Re
                 ]
             })
             .chain(untracked)
-            .chain(if unmerged.is_empty() {
-                vec![]
-            } else {
-                vec![
-                    items::blank_line(),
-                    Item {
-                        id: "unmerged".into(),
-                        display: Line::styled("Unmerged", &style.section_header),
-                        section: true,
-                        depth: 0,
-                        ..Default::default()
-                    },
-                ]
-            })
-            .chain(unmerged)
             .chain(create_status_section_items(
                 Rc::clone(&config),
                 "unstaged_changes",
                 Some(TargetData::AllUnstaged),
-                &git::diff_unstaged(&config, repo.as_ref())?,
+                &Rc::new(git::diff_unstaged(repo.as_ref())?),
             ))
             .chain(create_status_section_items(
                 Rc::clone(&config),
                 "staged_changes",
                 Some(TargetData::AllStaged),
-                &git::diff_staged(&config, repo.as_ref())?,
+                &Rc::new(git::diff_staged(repo.as_ref())?),
             ))
             .chain(create_stash_list_section_items(
                 Rc::clone(&config),
@@ -190,7 +171,9 @@ fn branch_status_items(config: &Config, repo: &Repository) -> Res<Vec<Item>> {
         return Ok(items);
     };
 
-    let (ahead, behind) = repo.graph_ahead_behind(head.target().unwrap(), upstream_id)?;
+    let (ahead, behind) = repo
+        .graph_ahead_behind(head.target().unwrap(), upstream_id)
+        .map_err(Error::GitStatus)?;
 
     items.push(Item {
         id: "branch_status".into(),
@@ -221,10 +204,10 @@ fn create_status_section_items<'a>(
     config: Rc<Config>,
     snake_case_header: &str,
     header_data: Option<TargetData>,
-    diff: &'a Diff,
+    diff: &'a Rc<Diff>,
 ) -> impl Iterator<Item = Item> + 'a {
     let style = &config.style;
-    if diff.deltas.is_empty() {
+    if diff.file_diffs.is_empty() {
         vec![]
     } else {
         vec![
@@ -241,7 +224,7 @@ fn create_status_section_items<'a>(
                         capitalize(&snake_case_header.replace("_", " ")),
                         &style.section_header,
                     ),
-                    format!(" ({})", diff.deltas.len()).into(),
+                    format!(" ({})", diff.file_diffs.len()).into(),
                 ]),
                 section: true,
                 depth: 0,
@@ -251,7 +234,7 @@ fn create_status_section_items<'a>(
         ]
     }
     .into_iter()
-    .chain(items::create_diff_items(config, diff, &1, true))
+    .chain(items::create_diff_items(config, diff, 1, true))
 }
 
 fn capitalize(str: &str) -> String {
