@@ -1,5 +1,5 @@
-use std::error::Error;
 use std::io::Read;
+use std::io::Write;
 use std::ops::DerefMut;
 use std::process::Child;
 use std::process::Command;
@@ -24,6 +24,7 @@ use crate::cli;
 use crate::cmd_log::CmdLog;
 use crate::cmd_log::CmdLogEntry;
 use crate::config::Config;
+use crate::error::Error;
 use crate::menu::Menu;
 use crate::menu::PendingMenu;
 use crate::ops::Op;
@@ -136,7 +137,8 @@ impl State {
         let needs_redraw = !events.is_empty() || pending_cmd_done;
 
         if needs_redraw && self.screens.last_mut().is_some() {
-            term.draw(|frame| ui::ui(frame, self))?;
+            term.draw(|frame| ui::ui(frame, self))
+                .map_err(Error::Term)?;
         }
 
         Ok(())
@@ -201,7 +203,7 @@ impl State {
         Ok(())
     }
 
-    fn handle_result<T>(&mut self, result: Result<T, Box<dyn Error>>) -> Option<T> {
+    fn handle_result<T>(&mut self, result: Res<T>) -> Option<T> {
         match result {
             Ok(value) => Some(value),
             Err(error) => {
@@ -248,7 +250,7 @@ impl State {
     /// Will return `Ok(())` if one is already running.
     pub fn run_cmd_async(&mut self, term: &mut Term, input: &[u8], mut cmd: Command) -> Res<()> {
         if self.pending_cmd.is_some() {
-            return Err("A command is already running".into());
+            return Err(Error::CmdAlreadyRunning);
         }
 
         cmd.current_dir(self.repo.workdir().expect("No workdir"));
@@ -258,12 +260,18 @@ impl State {
         cmd.stderr(Stdio::piped());
 
         let log_entry = self.current_cmd_log.push_cmd(&cmd);
-        term.draw(|frame| ui::ui(frame, self))?;
+        term.draw(|frame| ui::ui(frame, self))
+            .map_err(Error::Term)?;
 
-        let mut child = cmd.spawn()?;
+        let mut child = cmd.spawn().map_err(Error::SpawnCmd)?;
 
         use std::io::Write;
-        child.stdin.take().unwrap().write_all(input)?;
+        child
+            .stdin
+            .take()
+            .unwrap()
+            .write_all(input)
+            .map_err(Error::Term)?;
 
         self.pending_cmd = Some((child, log_entry));
 
@@ -276,7 +284,7 @@ impl State {
 
     fn await_pending_cmd(&mut self) -> Res<()> {
         if let Some((child, _)) = &mut self.pending_cmd {
-            child.wait()?;
+            child.wait().map_err(Error::CouldntAwaitCmd)?;
         }
         Ok(())
     }
@@ -287,7 +295,7 @@ impl State {
             return Ok(false);
         };
 
-        let Some(status) = child.try_wait()? else {
+        let Some(status) = child.try_wait().map_err(Error::CouldntAwaitCmd)? else {
             return Ok(false);
         };
 
@@ -303,10 +311,15 @@ impl State {
 
     pub fn run_cmd_interactive(&mut self, term: &mut Term, mut cmd: Command) -> Res<()> {
         if self.pending_cmd.is_some() {
-            return Err("A command is already running".into());
+            return Err(Error::CmdAlreadyRunning);
         }
 
-        cmd.current_dir(self.repo.workdir().expect("No workdir"));
+        cmd.current_dir(self.repo.workdir().ok_or(Error::NoRepoWorkdir)?);
+
+        self.current_cmd_log.push_cmd_with_output(&cmd, "\n".into());
+        self.update(term, &[GituEvent::Refresh])?;
+
+        eprint!("\r");
 
         // Redirect stderr so we can capture it via `Child::wait_with_output()`
         cmd.stderr(Stdio::piped());
@@ -317,38 +330,54 @@ impl State {
 
         // If we don't show the cursor prior spawning (thus restore the default
         // state), the cursor may be missing in $EDITOR.
-        term.show_cursor()?;
+        term.show_cursor().map_err(Error::Term)?;
 
-        let child = cmd.spawn()?;
+        let mut child = cmd.spawn().map_err(Error::SpawnCmd)?;
 
-        let out = child.wait_with_output()?;
-        let out_utf8 = String::from_utf8(strip_ansi_escapes::strip(out.stderr.clone()))
+        // Drop stdin as `Child::wait_with_output` would
+        drop(child.stdin.take());
+
+        let (mut stdout, mut stderr) = (Vec::new(), Vec::new());
+
+        tee(child.stdout.as_mut(), &mut [&mut stdout]).map_err(Error::Term)?;
+
+        tee(
+            child.stderr.as_mut(),
+            &mut [&mut std::io::stderr(), &mut stderr],
+        )
+        .map_err(Error::Term)?;
+
+        let status = child.wait().map_err(Error::CouldntAwaitCmd)?;
+        let out_utf8 = String::from_utf8(strip_ansi_escapes::strip(stderr.clone()))
             .expect("Error turning command output to String")
             .into();
 
+        self.current_cmd_log.clear();
         self.current_cmd_log.push_cmd_with_output(&cmd, out_utf8);
 
         // restore the raw mode
         term.backend().enable_raw_mode()?;
 
         // Prevents cursor flash when exiting editor
-        term.hide_cursor()?;
+        term.hide_cursor().map_err(Error::Term)?;
 
         // In case the command left the alternate screen (editors would)
         term.backend_mut().enter_alternate_screen()?;
 
-        term.clear()?;
+        term.clear().map_err(Error::Term)?;
         self.screen_mut().update()?;
 
-        if !out.status.success() {
-            return Err(format!(
-                "exited with code: {}",
-                out.status
-                    .code()
-                    .map(|c| c.to_string())
-                    .unwrap_or("".to_string())
-            )
-            .into());
+        if !status.success() {
+            return Err(Error::CmdBadExit(
+                format!(
+                    "{} {}",
+                    cmd.get_program().to_string_lossy(),
+                    cmd.get_args()
+                        .map(|arg| arg.to_string_lossy())
+                        .collect::<String>()
+                ),
+                status.code(),
+            ));
         }
 
         Ok(())
@@ -367,6 +396,28 @@ impl State {
     }
 }
 
+fn tee(maybe_input: Option<&mut impl Read>, outputs: &mut [&mut dyn Write]) -> std::io::Result<()> {
+    let Some(input) = maybe_input else {
+        return Ok(());
+    };
+
+    let mut buf = [0u8; 1024];
+
+    loop {
+        let num_read = input.read(&mut buf)?;
+        if num_read == 0 {
+            break;
+        }
+
+        let buf = &buf[..num_read];
+        for output in &mut *outputs {
+            output.write_all(buf)?;
+        }
+    }
+
+    Ok(())
+}
+
 pub(crate) fn root_menu(config: &Config) -> Option<Menu> {
     if config.general.always_show_help.enabled {
         Some(Menu::Help)
@@ -379,7 +430,7 @@ fn write_child_output_to_log(
     log_rwlock: &mut Arc<RwLock<CmdLogEntry>>,
     child: &mut Child,
     status: std::process::ExitStatus,
-) -> Result<(), Box<dyn Error>> {
+) -> Res<()> {
     let mut log = log_rwlock.write().unwrap();
 
     let CmdLogEntry::Cmd { args, out: out_log } = log.deref_mut() else {
@@ -396,28 +447,20 @@ fn write_child_output_to_log(
         .take()
         .unwrap()
         .read_to_end(&mut out_bytes)
-        .map_err(|e| format!("Couldn't read cmd output: {}", e))?;
+        .map_err(Error::CouldntReadCmdOutput)?;
 
     child
         .stdout
         .take()
         .unwrap()
         .read_to_end(&mut out_bytes)
-        .map_err(|e| format!("Couldn't read cmd output: {}", e))?;
+        .map_err(Error::CouldntReadCmdOutput)?;
 
-    let out_string = String::from_utf8(out_bytes.clone())?;
+    let out_string = String::from_utf8_lossy(&out_bytes).to_string();
     *out_log = Some(out_string.into());
 
     if !status.success() {
-        return Err(format!(
-            "'{}' exited with code: {}",
-            args,
-            status
-                .code()
-                .map(|c| c.to_string())
-                .unwrap_or("".to_string())
-        )
-        .into());
+        return Err(Error::CmdBadExit(args.to_string(), status.code()));
     }
 
     Ok(())
